( cd usr/lib ; rm -rf libgaim-remote.so.0 )
( cd usr/lib ; ln -sf libgaim-remote.so.0.0.0 libgaim-remote.so.0 )
( cd usr/lib ; rm -rf libgaim-remote.so )
( cd usr/lib ; ln -sf libgaim-remote.so.0.0.0 libgaim-remote.so )
